/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2009                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include<vcg/simplex/vertex/base.h>
#include<vcg/simplex/vertex/component.h>

#include <vcg/complex/used_types.h>

#include<vcg/simplex/face/base.h>
#include<vcg/simplex/face/component.h>
#include<vcg/complex/complex.h>
#include<vcg/complex/algorithms/create/platonic.h>
#include<wrap/io_trimesh/import_ply.h>
#include<wrap/io_trimesh/export_ply.h>

using namespace vcg;
using namespace std;

class MyEdge;
class MyFace;
class MyVertex;
struct MyUsedTypes : public UsedTypes<	Use<MyVertex>   ::AsVertexType,
                                        Use<MyEdge>     ::AsEdgeType,
                                        Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex<MyUsedTypes, vertex::Coord3f, vertex::Normal3f, vertex::BitFlags  >{};
class MyFace    : public Face< MyUsedTypes, face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<MyUsedTypes>{};
class MyMesh    : public tri::TriMesh< vector<MyVertex>, vector<MyFace> , vector<MyEdge>  > {};

class PartialSum
{
public:
  PartialSum()
  {
    cnt=0;
    pos=Point3f(0,0,0);
  }

  int cnt;
  Point3f pos;
};


int main( int argc, char **argv )
{
  MyMesh m;
  if(argc < 3 ) return -1;

  printf("Reading %s\n",argv[1]);
  int ret= tri::io::ImporterPLY<MyMesh>::Open(m,argv[1]);
  if(ret!=0)
  {
    printf("Unable to open %s for '%s'\n",argv[1],tri::io::ImporterPLY<MyMesh>::ErrorMsg(ret));
    return -1;
  }
  printf("Mesh has %i vert and %i faces\n",m.vn, m.fn);
  int SmoothingStep=atoi(argv[2]);

  tri::UpdateFlags<MyMesh>::VertexBorderFromNone(m);

  MyMesh::PerVertexAttributeHandle<PartialSum> ih = tri::Allocator<MyMesh>::AddPerVertexAttribute<PartialSum>(m);
  for(int j=0;j<SmoothingStep;++j)
  {
    for(MyMesh::FaceIterator fi=m.face.begin();fi!=m.face.end();++fi)
    {
      for(int i=0;i<3;++i)
      {
        ih[fi->V(i)].pos += fi->V1(i)->P();
        ih[fi->V(i)].pos += fi->V2(i)->P();
        ++ih[fi->V(i)].cnt;
      }
    }
    float alpha = 0.5;
    for(MyMesh::VertexIterator vi=m.vert.begin();vi!=m.vert.end();++vi)
    {
      if(!vi->IsB())
      {
        Point3f barycenter = ih[vi].pos * (0.5f / float(ih[vi].cnt));
        vi->P() += (barycenter-vi->P()) * alpha ;
        ih[vi].pos = Point3f(0,0,0);
        ih[vi].cnt = 0;
      }
    }
  }
  tri::io::ExporterPLY<MyMesh>::Save(m,"out.ply");

  return 0;
}
